-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity

-- [[ DGC v1.2 Multi Character Selection ]]--

local addonName, addonTable = ...
local foreach = table.foreach
local tx_cross = "Interface\\RaidFrame\\ReadyCheck-NotReady"

local L = addonTable.L
local Region = GetCurrentRegion() --GetCVar("portal")

local delimiter = "~"

local seconds_per_hour = 60 * 60
local seconds_per_day = seconds_per_hour * 24

local chars = DailyGlobalCheck.CharFunctions
local quest = DailyGlobalCheck.QuestFunctions

local pID = addonTable.pID

function chars:isquestcompleted(questID, character)
 local c = DailyGlobalCheck_CharData[character]
 local qData = DailyGlobalCheck:getQData(questID)
 if not c or not c.data or not c.data[questID] then return false end

 -- old version update
 if type(c.data[questID]) == "table" then
  c.data[questID] = (c.data[questID][2] or "")..delimiter
 end

 local cstring, reset = strsplit(delimiter, c.data[questID])

 if not qData or chars:isOutDated(qData[7], reset and tonumber(reset) or c.resets) then
  c.data[questID] = nil
  return false
 end
 
 return true, cstring ~= "" and cstring or nil
end

-- packs a quest as a string with this format
-- cstring ~ personal reset time ~ tooltip info
function chars:PackQuest(questID, cstring, reset)
 local qData = DailyGlobalCheck:getQData(questID)
 local result = (cstring or "")..delimiter

 if reset then -- special quest
  local link = quest:GetOverlayInfo(questID)
  result = result..(time() + reset)..delimiter
  if link then
   result = result..link
  end
 end

 return result
end

local function calculateResets()

 local weekly_reset_time = { hour = 8, min = 0 } -- not 100% sure.. but it should be maintenance time
 local weekly_reset_day = Region == 1 and 3 or 4

 local t = time()
 local currDate = date("*t")
 local weeklyReset = t
 local dailyReset = t + GetQuestResetTime()

 local wd = currDate.wday
 local i = currDate.hour

 while i ~= weekly_reset_time.hour do
  if i < weekly_reset_time.hour then
   weeklyReset = weeklyReset + seconds_per_hour
   i = i + 1
  else
   weeklyReset = weeklyReset - seconds_per_hour
   i = i - 1
  end
 end
 
 -- subtract the remaining minutes to reach 8:00
 weeklyReset = weeklyReset - currDate.min * 60
 
 i = wd
 while i ~= weekly_reset_day do
  i = i + 1
  weeklyReset = weeklyReset + seconds_per_day
  if i > 7 then i = 1 end
 end
 
 return weeklyReset, dailyReset
end

function chars:isOutDated(qType, t)
 -- normal quests are never outdated
 if qType == "Q" then return false end
 
 local now = time()

 if type(t) == "number" then return now > t end
 
 local dReset = t.dReset or 0
 local wReset = t.wReset or 0
 local mReset = t.mReset or 0

 if qType == "W" then -- resets at maintenance time (weekly)
  return now > wReset
 elseif qType == "M" then -- resets at midnight
  return now > mReset
 else -- resets at standard daily reset time (3:00 AM)
  return now > dReset
 end
end

function chars:SaveCharData()

 if tContains(DailyGlobalCheck_Options["chars_blacklist"], pID) then
  return
 end
-- /run DailyGlobalCheck.CharFunctions:SaveCharData()
 local result = {}
 result.data = {}

 DailyGlobalCheck:foreachLists(function(k,v)
  if v.MultiCharsEnabled then
   for _,o in pairs(v["Order"]) do
    for _,questIDs in pairs(o) do
     for index,qID in pairs(questIDs) do
      if index > 1 then -- 1 is the header
 	   local b, cstring, reset = addonTable.isquestcompleted(qID, true, v)
	   if b then
        result.data[qID] = chars:PackQuest(qID, cstring, reset)
	   end
	  end
     end
    end
   end
  end
 end)
 
 local wr, dr = calculateResets()
 local mr = dr - seconds_per_hour * 3
 result.resets = { wReset = wr, dReset = dr, mReset = mr }

 --[[ debug
 local tmp = date("*t", wr)
 print("weekly reset")
 print(tmp.day.."/"..tmp.month.."/"..tmp.year.." ("..tmp.wday..") - "..tmp.hour..":"..tmp.min)
 
 tmp = date("*t", dr)
 print("daily reset")
 print(tmp.day.."/"..tmp.month.."/"..tmp.year.." ("..tmp.wday..") - "..tmp.hour..":"..tmp.min)
 ]]
 
 DailyGlobalCheck_CharData[pID] = result
end

local frames_backdrop  = { bgFile = "Interface\\Buttons\\GreyscaleRamp64", 
                           edgeFile = "Interface\\Buttons\\WHITE8X8", 
						   edgeSize = 1 }

function chars:CreateCharactersFrame(parent)

 DailyGlobalCheck.CharactersFrame = CreateFrame("Frame", "DGCCharactersFrame", parent)
 local f = DailyGlobalCheck.CharactersFrame

  local function repositionbuttons()
   local w,h,counter = 300,0,0
   foreach(f.buttons, function(k,v)
	v:SetPoint("TOPLEFT", f, "TOPLEFT", 2, -(2 + counter * 20))
    counter = counter + 1
    h = h + 20
   end)
   f:SetSize(w,h)
  end
 
  local function btnselect(self)
   if DailyGlobalCheck.selectedpID == self.pID then return end
   
   foreach(f.buttons, function(_,v) v:SetBackdropColor(0.1,0.1,0.1,1) end)
   self:SetBackdropColor(0,0.3,0,1)
   DailyGlobalCheck.selectedpID = self.pID
   addonTable.refresh()
  end

  local function deleteChar(btn)
   local i = -1
   foreach(f.buttons, function(k,v) if v == btn then i = k return end end)
   if i > -1 then
    table.remove(f.buttons, i)
	if DailyGlobalCheck_CharData[btn.pID] then DailyGlobalCheck_CharData[btn.pID] = nil end
	btn:Hide()
	repositionbuttons()
	if not tContains(DailyGlobalCheck_Options["chars_blacklist"], btn.pID) then
	 table.insert(DailyGlobalCheck_Options["chars_blacklist"], btn.pID)
	end
	print(string.format(L["chars_deleted"], btn.pID))
   end
  end
 
  local function createCharButton(pid, isPlayer)
   if (not isPlayer) and (pid == pID or
      tContains(DailyGlobalCheck_Options["chars_blacklist"], pid)) then return end
   
   local btn = CreateFrame("Frame", nil, f)
 
   btn:SetBackdrop(frames_backdrop)
   btn:SetBackdropColor(0.1,0.1,0.1,1)
   btn:SetBackdropBorderColor(0.2,0.2,0.2,1)
   btn:SetFrameStrata("MEDIUM")
   btn:SetSize(300, 20)
   
   btn.text = btn:CreateFontString(nil, "OVERLAY", "GameTooltipText")
   btn.text:SetAllPoints()
   btn.text:SetText(pid)

   btn:SetScript("OnEnter", function()
    btn:SetBackdropBorderColor(0,0.8,0.8,1)
   end)
   btn:SetScript("OnLeave", function()
    btn:SetBackdropBorderColor(0.2,0.2,0.2,1)
   end)
   btn:SetScript("OnMouseUp", btnselect)
   
   btn.pID = pid
   
   if DailyGlobalCheck.selectedpID == pid then
    btn:SetBackdropColor(0,0.3,0,1)
   end
   
   if pid ~= pID then
    local delbtn = CreateFrame("Frame", nil, btn)
    delbtn:SetSize(20,20)
    delbtn:SetPoint("TOPRIGHT")
    delbtn.texture = delbtn:CreateTexture()
    delbtn.texture:SetAllPoints()
    delbtn.texture:SetTexture(tx_cross)
    delbtn:SetScript("OnMouseUp", function() deleteChar(btn) end)
   end
   
   table.insert(f.buttons, btn)
  end

 f.buttons = {}
 f:SetPoint("TOP", parent, "BOTTOM")
 f:SetBackdrop(frames_backdrop)
 f:SetBackdropColor(0.07,0.07,0.07,0.9)
 f:SetBackdropBorderColor(0.07,0.07,0.07,0.9)
 f:SetFrameStrata("MEDIUM")

 createCharButton(pID, true)
 foreach(DailyGlobalCheck_CharData, function(k) createCharButton(k) end)
 repositionbuttons()

 f:Hide()
 
 return f
end

local logoutfunc = Logout
function Logout()
 if chars then chars:SaveCharData() end
 logoutfunc()
end

local quitfunc = Quit
function Quit()
 if chars then chars:SaveCharData() end
 quitfunc()
end

--[[
DGCEventFrame:RegisterEvent("PLAYER_LEAVING_WORLD")
function DGCEventFrame.PLAYER_LEAVING_WORLD(...)
 chars:SaveCharData()
end
]]